/* ZAPI.H - ZAPI User Functions and data structs
   Copyright (c) 1995 Sangoma Technologies Inc.
   Author: Glenn Robins
   API v1.0, 16-bit

   Must #include <windows.h> before this header
*/

#define	MDATALEN	4108
#define	CB_HEADERLEN	15

/* control block */
typedef struct {
	BYTE command;
	WORD buffer_length;
	BYTE return_code;
	WORD no_rx_frms_discarded_int_lvl;
	WORD no_rx_frms_discarded_app_lvl;
	WORD rx_frame_time_stamp;
	BYTE misc_tx_rx_bits;
	WORD no_bytes_avail_in_tx_bfr;
	BYTE reserved[2];
	BYTE data[MDATALEN];
} CBLOCK;


/* ZAPI will always return 1 */
int WINAPI ZAPIAlive(void);

/* Initializes ZAPI and loads SDLA driver, if not loaded already.
   A trace should be registered before this setup if one will
   be used at all, but is not necessary.
   Each application instance MUST call this on initialization.
   returns 0 if failed to load SDLA driver,
	   1 if this is the first setup call and the SDLA driver
	     loaded successfully,
	   2 if this is not the first successful setup call
	     (driver already loaded)
*/
int WINAPI ZAPISetup(void);

/* Cleanup and unloads SDLA driver if this is the last
   application instance to use the driver.
   You must terminate your trace registration first if one
   was registered, unless you are the last application to use
   this DLL (but you shouldn't assume this).
   Each application instance MUST call this before closing.
   returns 1 always (return code reserved for future use)
*/
int WINAPI ZAPICleanup(void);

/* Emulate FRIP interrupt call with control block
   returns return_code value in structure
*/
char WINAPI ZAPIControl(CBLOCK far *block, BYTE cardnumber);

/* returns 0 if a debugging command trace is already registered or
   had difficulty creating the window, 1 if successful.
   hWnd is a window to send notification messages to, and
   iMessage is a user-defined message that ZAPI will send the
   application to signal an ZAPI notification. The actual
   message code is passed by value to the WPARAM parameter
   of the window procedure
*/
char WINAPI ZAPIRegisterDebug(HINSTANCE hInstance, HWND hWnd, UINT iMessage);

/* if a debugging command trace is active and instance matches that passed
   when registered (for security/fault tolerance purposes) then
   trace is terminated and returns 0. If instance doesn't match,
   returns 1. If no trace is running, returns 2.
   The trace is also terminated automatically if the window is
   closed (see trace messages)
*/
char WINAPI ZAPITerminateDebug(HINSTANCE hInstance);

/* returns TRUE if a debugging command trace is registered and active */
BOOL WINAPI ZAPIIsDebugRegistered(void);

/* DEBUGGING COMMAND TRACE MESSAGES (wParam values) */
#define	TRACEMSG_CLOSE	0

/*
   returns SDLAR_NOT_REGISTERED (0L) if failed to obtain a selector
	(any read or write calls to the board will result
	in a general protection fault)
   returns SDLAR_REGISTERED if succeeded (1L): the card's
	instance count was incremented but this is not the
	first registration call for this board (it is already
	in use by an application)
   returns SDLAR_ALLOCATED if succeeded (2L): the card's
	instance count was incremented and this is the
	first registration call for this board, and the
	memory selector was allocated
*/
int WINAPI ZAPIRegisterCard(BYTE cardnumber);

/*
   returns SDLAR_NOT_REGISTERED (0L) if card wasn't registered
   returns SDLAR_DEREGISTERED (1L) if card's instance count
	was decremented but isn't currently zero
	(i.e. an application is still using the specified board)
   returns SDLAR_RELEASED (2L) if card is now not used by
	any application, and the selector was freed
*/
int WINAPI ZAPIDeregisterCard(BYTE cardnumber);

/*
   returns a bit record (32-bit) of cards registered for use
	and cards that exist:
	[ E15 E14 ... E3 E2 E1 E0 R15 R14 ... R3 R2 R1 R0 ]
   For example, if bit R2 is set, then card 2 is registered
		if bit E2 is set, then card 2 exists
			(i.e. a configuration entry exists
			in WIN.INI)
*/
LRESULT WINAPI ZAPIGetCards(void);

/* returns a byte indicating current modem status as follows:
	[ 0 0 CTS 0 DCD 0 0 0 ]
   CTS and DCD are high when these corresponding bits are set
*/
char WINAPI ZAPIGetModemStatus(BYTE cardnumber);

/* returns 0x01 if at least one frame is queued for transmission, 0x00 if not */
char WINAPI ZAPIqQueuedToXmit(BYTE cardnumber);
/* returns 0x01 if at least one frame is queued for reception, 0x00 if not */
char WINAPI ZAPIqDataToRead(BYTE cardnumber);

/*
   returns 0 if nothing in receive buffer
   returns 1 if received a frame which is stored in *block
   requires ZAPIEnableFastRead() to be called first
*/
int WINAPI ZAPIFastRead(CBLOCK far *block, BYTE cardnumber);

/*
	returns 0 if succeeded
	returns return_code of SET_INTERRUPT_TRIGGERS otherwise
*/
char WINAPI ZAPIEnableFastRead(BYTE cardnumber);

/*
	returns 0 if succeeded
	returns return_code of SET_INTERRUPT_TRIGGERS otherwise
*/
char WINAPI ZAPIDisableFastRead(BYTE cardnumber);


int WINAPI ZAPIPrepareForLoading(BYTE cardnumber);
int WINAPI ZAPILoadComplete(BYTE cardnumber);
unsigned char WINAPI ZAPIReadPortByte(BYTE cardnumber, WORD offset);
void WINAPI ZAPIWritePortByte(BYTE cardnumber, WORD offset, BYTE data);
unsigned char WINAPI ZAPIReadByte(BYTE cardnumber, WORD offset);
void WINAPI ZAPIWriteByte(BYTE cardnumber, WORD offset, BYTE data);
void WINAPI ZAPIZeroWindow(BYTE cardnumber);

