/*
   ZTEST.C - Sample program for ZAPI.DLL
   Copyright (c) 1995 Sangoma Technologies Inc.
   Author: Glenn Robins
   November 1995

   Use at your own risk.
   Build:
        (Static) runtime library only, plus ZAPI.LIB or ZAPI32.LIB
	Windows 16-bit .EXE, Large Model
	or Win32 GUI

*/

/* for "portability" to Win32 */
#define STRICT

// define MAKEFORNT if building for Win32/WinNT
/*
#define MAKEFORNT
*/

#include <windows.h>
#include <windowsx.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <commdlg.h>
#include <io.h>
#include <fcntl.h>
#include <sys\stat.h>
#include "\zapi\zapi.h"
#include "\zapi\zapi_m.h"


#define SAYRESPONSE(s)	SetDlgItemText(exehwn, 900, s)
#define	SAYSTATE(x)	SetWindowText(exehwn, x)
#define	SAYNOSTATE()	SetWindowText(exehwn, szProgramHeader)

#define	IDM_ABOUT	0x1000

#define STATS_TIMER	3000

HINSTANCE ghinstance;
HWND exehwn;
HWND rxdhwn = NULL;
HWND statshwn = NULL;
DLGPROC dprocrd, dprocst;
CBLOCK cb;

char txtext[1000];
char rxdetail_text[16384];

struct {
	BYTE line_speed;
	WORD max_frame_length;
	BYTE percent_tx_bfrs;
	BYTE config_bits;
	BYTE modem_bits;
	BYTE stat_bits;
	BYTE bit_times_open;
	BYTE bit_times_close;
	BYTE cts_rts_delay;
} cfg;

char *Sspeeds[] = { "External", "1200", "2400", "4800", "9600", "19200", "38400", "45000", "56000", "74000", "112000", "128000", "156000", "192000", "" };


char *our_ini_file = "ztest.ini";
char *iniHardwareSection = "Hardware";
char *iniCardEntry = "default_card_number";
char *iniAskEntry = "ask_for_card_on_startup";

char *szhigh = "HIGH";
char *szlow = "LOW";
char *szstop = "Stop!";
char *szwritestring = "Write String >>";
char *szreadmany = "Read Many";

#ifdef MAKEFORNT
char *abouttitle = "About The ZTESTer v1.0nt";
char *aboutstatement =
"ZTEST for NT is Copyright (c) 1995 Sangoma Technologies Inc.\nNovember 1995 release";
#else
char *abouttitle = "About The ZTESTer v1.0";
char *aboutstatement =
"ZTEST is Copyright (c) 1995 Sangoma Technologies Inc.\nNovember 1995 release";
#endif
char szProgramHeader[60];
BYTE card = 0;
BYTE askonstartup = 0;

BYTE writestringflag = 0;
BYTE readmanyflag = 0;
WORD num_write_frames;
WORD write_frame_length;
BYTE write_byte;
BYTE write_inc_flag;
BYTE write_bits;
WORD initial_write_frames;

char *logfilter = "Cfg Files (*.cfg)\0*.cfg\0All Files (*.*)\0*.*\0";

char *szTitleName = "Basic Functions Streaming Tester";

/* prototypes */
int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int cmdShow);
LRESULT CALLBACK _export MainWndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM lParam );
LRESULT CALLBACK _export SelCardDlg(HWND hDlg, UINT iMessage, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK _export RxDetailDlg(HWND hDlg, UINT iMessage, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK _export StatsDlg(HWND hDlg, UINT iMessage, WPARAM wParam, LPARAM lParam);
void DoCommand(void);
BOOL FirstSelCard(void);
BOOL ParseCmdLineForCard(LPSTR lpszCmdLine);
int ParseTextToHex(int l);
void SelCard(void);
void GetConfig(void);
void DoStopWrite(void);
void DoWriteString(void);
void DoReadMany(void);
void WriteConfig(void);
void LoadConfig(void);
void PutConfig(void);
void UpdateAllStats(void);

int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int cmdShow)
{
	MSG msg;
	BOOL fRetVal;
	WNDCLASS wndclass;
	HMENU hmenu;
	char s[120];
	WORD i;

	ghinstance = hInstance;

	// get startup card info
	card = (BYTE)GetPrivateProfileInt(iniHardwareSection, iniCardEntry, 0, our_ini_file);
	askonstartup = (BYTE)GetPrivateProfileInt(iniHardwareSection, iniAskEntry, 0, our_ini_file);

	if (hPrevInstance == NULL) {

		/* set up and register the window class */
		wndclass.style         = CS_HREDRAW | CS_VREDRAW;
		wndclass.lpfnWndProc   = MainWndProc;
		wndclass.cbClsExtra    = 0;
		wndclass.cbWndExtra    = DLGWINDOWEXTRA;
		wndclass.hInstance     = ghinstance;
		wndclass.hIcon         = LoadIcon(ghinstance, "tester");
		wndclass.hCursor       = LoadCursor(NULL, IDC_ARROW);
		wndclass.hbrBackground = (HBRUSH)GetStockObject(GRAY_BRUSH);
		wndclass.lpszMenuName  = NULL;
		wndclass.lpszClassName = "ZTEST";

		if (!RegisterClass(&wndclass)) {
			return(0);
		}

	} // end instance check

	exehwn = CreateDialog(ghinstance, "MAINDLG", HWND_DESKTOP, NULL);

	if (!exehwn)  return(0);	/* failed to create window */

	// must tell ZAPI to initialize SDLA driver if we are first
	ZAPISetup();

	// see if user wants to override startup card
	if (!ParseCmdLineForCard(lpszCmdLine)) {
		if (askonstartup) {
			if (!FirstSelCard()) {
				DestroyWindow(exehwn);
				ZAPICleanup();
				return(0);
			}
		}
	}

	// must register for the use of 'card' before we can use it
	if (!ZAPIRegisterCard(card)) {
		sprintf(s, "Unable to register card %u!  ZTEST will now terminate.  (Default card number may be changed by editing ZTEST.INI.)", card);
		MessageBox(exehwn, s, "ZTEST FATAL ERROR", MB_OK | MB_ICONHAND);
		DestroyWindow(exehwn);
		ZAPICleanup();
		return(0);
	}

	dprocrd = (DLGPROC)MakeProcInstance((FARPROC)RxDetailDlg, ghinstance);
	dprocst = (DLGPROC)MakeProcInstance((FARPROC)StatsDlg, ghinstance);

	hmenu = GetSystemMenu(exehwn, FALSE);
	AppendMenu(hmenu, MF_SEPARATOR, 0, (LPSTR) NULL);
	AppendMenu(hmenu, MF_STRING, IDM_ABOUT, "About...");

	sprintf(szProgramHeader, "%s (Card %u)", szTitleName, card);
	SAYNOSTATE();

	// set default values in window
	// line speed
	SendDlgItemMessage(exehwn, 800, CB_RESETCONTENT, 0, 0L);
	i=0;
	while (Sspeeds[i][0] != NULL) {
		sprintf(s, "%s", Sspeeds[i]);
		SendDlgItemMessage(exehwn, 800, CB_ADDSTRING, 0, (LPARAM)s);
		i++;
	}
	SendDlgItemMessage(exehwn, 800, CB_SETCURSEL, 0, 0L);

	SetDlgItemText(exehwn, 801, "4096");
	SetDlgItemText(exehwn, 802, "50");
	SetDlgItemText(exehwn, 803, "0");
	SetDlgItemText(exehwn, 804, "0");
	SetDlgItemText(exehwn, 805, "0");
	SetDlgItemText(exehwn, 912, "00");
	SetDlgItemText(exehwn, 913, "1");
	SetDlgItemText(exehwn, 914, "1");
	CheckDlgButton(exehwn, 400, 1);
	CheckDlgButton(exehwn, 401, 1);

	fRetVal = TRUE;		/* keep looping until FALSE (WM_QUIT) */

	/* stay in this outside loop until the WM_QUIT message has
	been encountered and processed */
	while (fRetVal)  {
		while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) {
			if (msg.message == WM_QUIT)
				fRetVal = FALSE;	/* leave main loop */
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		// perform continuous read/write ops if any
		// (this is a real system hog)
		if (readmanyflag)
			DoReadMany();
		if (writestringflag)
			DoWriteString();
	}
	// User wants to quit so clean things up
	if (rxdhwn != NULL)
		DestroyWindow(rxdhwn);
	if (statshwn != NULL) {
		KillTimer(statshwn, 1);
		DestroyWindow(statshwn);
	}
	FreeProcInstance((FARPROC)dprocrd);
	FreeProcInstance((FARPROC)dprocst);
	// clean ZAPI things up - VERY IMPORTANT!!!
	ZAPIDeregisterCard(card);
	ZAPICleanup();
	return fRetVal;
}

BOOL ParseCmdLineForCard(LPSTR lpszCmdLine)
{
	int i;
	BYTE c;

	i=0;
	while (lpszCmdLine[i] != NULL && lpszCmdLine[i] != '-') i++;
	if (lpszCmdLine[i] == '-') {
		c = (BYTE)atoi((LPSTR)(lpszCmdLine+i+1));
		if (c < 16) {
			card = c;
			return(TRUE);
		}
	}
	return(FALSE);
}

BOOL FirstSelCard(void)
{
	DLGPROC dproc;
	int i;

	dproc = (DLGPROC)MakeProcInstance((FARPROC)SelCardDlg, ghinstance);
	i=DialogBox(ghinstance, "SELCARD", exehwn, dproc);
	FreeProcInstance((FARPROC)dproc);
	if (i) {
		card = i-1;
		return(TRUE);
	}
	return(FALSE);
}

void SelCard(void)
{
	DLGPROC dproc;
	int i;
//	char s[80];

	dproc = (DLGPROC)MakeProcInstance((FARPROC)SelCardDlg, ghinstance);
	i=DialogBox(ghinstance, "SELCARD", exehwn, dproc);
	FreeProcInstance((FARPROC)dproc);
	if (i) {
		if (ZAPIRegisterCard(i-1)) {
			ZAPIDeregisterCard(card);
			card = i-1;
			sprintf(szProgramHeader, "%s (Card %u)", szTitleName, card);
			SAYNOSTATE();
		}
		else {
			MessageBox(exehwn, "Unable to register specified card!", "FTEST FATAL ERROR", MB_OK | MB_ICONHAND);
		}
	}
}

/* main window procedure */
LRESULT CALLBACK _export MainWndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM lParam )
{
	WORD v;
	char s[5];

	switch(iMessage) {
	case WM_COMMAND:
		switch(GET_WM_COMMAND_ID(wParam, lParam)) {
		case 100:	// Exit button
			DestroyWindow(hWnd);
			PostQuitMessage(0);
			return(0);
		case 101:	// Select card button
			SelCard();
			return(0);
		case 200:	// ENABLE
			cb.command = ENABLE_COMMUNICATIONS;
			cb.buffer_length = 0;
			DoCommand();
			return(0);
		case 201:	// DISABLE
			cb.command = DISABLE_COMMUNICATIONS;
			cb.buffer_length = 0;
			DoCommand();
			return(0);
		case 202:	// READ_MODEM_STATUS
			// do it the command way since it fits into this
			// scheme more easily - it's better to use
			// ZAPIGetModemStatus normally
			cb.command = READ_MODEM_STATUS;
			cb.buffer_length = 0;
			DoCommand();
			return(0);
		case 203:	// MODEM CONTROL
			cb.command = SET_MODEM_STATUS;
			cb.buffer_length = 1;
			cb.data[0] = (IsDlgButtonChecked(exehwn, 400)) ? 0x01 : 0x00;
			cb.data[0] |= (IsDlgButtonChecked(exehwn, 401)) ? 0x02 : 0x00;
			DoCommand();
			return(0);
		case 205:	// BRIDGE RX/TX
			cb.command = BRIDGE_RX_AND_TX;
			cb.buffer_length = 1;
			cb.data[0] = (IsDlgButtonChecked(exehwn, 405)) ? 0x01 : 0x00;
			DoCommand();
			return(0);
		case 220:	// READ MANY
			if (readmanyflag) {
				SetDlgItemText(exehwn, 220, szreadmany);
				readmanyflag = 0;
			} else {
				SetDlgItemText(exehwn, 220, szstop);
				readmanyflag = 1;
			}
			return(0);
		case 221:
			if (rxdhwn == NULL) {
				rxdhwn=CreateDialog(ghinstance, "RDETAIL", exehwn, dprocrd);
			}
			return(0);
		case 204:	// open stats window
			if (statshwn == NULL) {
				statshwn=CreateDialog(ghinstance, "STATS", exehwn, dprocst);
				if (SetTimer(statshwn, 1, STATS_TIMER, NULL) == 0)
					MessageBox(exehwn, "Warning: no system timers are available so statistics must be updated manually.", "No Timer Available", MB_OK | MB_ICONEXCLAMATION);
			}
			return(0);
		case 210:	// READ
			cb.command = RECEIVE_FRAME;
			cb.buffer_length = 0;
			DoCommand();
			return(0);
		case 211:	// WRITE
			cb.command = SEND_FRAME;
			v = GetDlgItemText(exehwn, 901, txtext, 1000);
			v = ParseTextToHex(v);
			cb.buffer_length = v;
			cb.misc_tx_rx_bits = 0;
			cb.misc_tx_rx_bits |= (IsDlgButtonChecked(exehwn, 403)) ? 0x01 : 0x00;
			cb.misc_tx_rx_bits |= (IsDlgButtonChecked(exehwn, 402)) ? 0x02 : 0x00;
			DoCommand();
			return(0);
		case 212:	// WRITE repeated constant
			if (writestringflag) {
				DoStopWrite();
			} else {
				write_frame_length = (WORD)GetDlgItemInt(exehwn, 913, NULL, FALSE);
				GetDlgItemText(exehwn, 912, s, 3);
				sscanf(s, "%2x", &v);
				write_byte = (BYTE)v;
				write_bits = 0;
				write_bits |= (IsDlgButtonChecked(exehwn, 403)) ? 0x01 : 0x00;
				write_bits |= (IsDlgButtonChecked(exehwn, 402)) ? 0x02 : 0x00;
				write_inc_flag = (IsDlgButtonChecked(exehwn, 404)) ? 0x01 : 0x00;
				num_write_frames = (WORD)GetDlgItemInt(exehwn, 914, NULL, FALSE);
				writestringflag = 1;
				SetDlgItemText(exehwn, 212, szstop);
				initial_write_frames = num_write_frames;
				EnableWindow(GetDlgItem(exehwn, 914), FALSE);
			}
			return(0);
		case 230:	// SET CONFIGURATION
			cb.command = SET_CONFIGURATION;
			cb.buffer_length = 0x0a;
			GetConfig();
			memcpy(cb.data, &cfg.line_speed, 0x0a);
			DoCommand();
			return(0);
		case 231:	// SAVE CONFIGURATION
			WriteConfig();
			return(0);
		case 232:	// LOAD CONFIGURATION
			LoadConfig();
			return(0);
		}
		break;

	case WM_SYSCOMMAND:
		if (wParam == SC_CLOSE) {
			DestroyWindow(hWnd);
			PostQuitMessage(0);
			return(0);
		}
		if (GET_WM_COMMAND_ID(wParam, lParam) == IDM_ABOUT) {
			MessageBox(exehwn, aboutstatement, abouttitle, MB_OK | MB_ICONINFORMATION);
			return(0);
		}
		break;
	}
	return DefDlgProc( hWnd, iMessage, wParam, lParam );
}

/*
	Performs command in CBLOCK cb, and displays return codes and
	specific return info if any
*/
void DoCommand(void)
{
	char s[150], s1[150], t[10];
	WORD i, n;

	ZAPIControl(&cb, card);
	sprintf(s, "Return code: 0x%02X - ", (BYTE)cb.return_code);
	switch(cb.return_code) {
	case 0x00:
		switch(cb.command) {
		case READ_COMMS_ERR_STATISTICS:
		case READ_OPERATIONAL_STATISTICS:
			strcat(s, "Statistics read successfully");
			break;
		case READ_MODEM_STATUS:
			sprintf(s1, "CTS is %s and DCD is %s", (cb.data[0] & 0x20) ? szhigh : szlow, (cb.data[0] & 0x08) ? szhigh : szlow);
			strcat(s, s1);
			break;
		case RECEIVE_FRAME:
			sprintf(s1, "%u bytes (%s): ", cb.buffer_length, (cb.misc_tx_rx_bits) ? "CRC Error " : "OK");
			strcat(s, s1);
			for (i=0; i < cb.buffer_length; i++) {
				if (i > 12) {
					strcat(s, "...");
					break;
				}
				sprintf(s1, "%02X ", (BYTE)cb.data[i]);
				strcat(s, s1);
			}
			if (rxdhwn != NULL) {
				rxdetail_text[0] = 0;
				s1[0] = 0;
				for (n=0; n < cb.buffer_length; n++) {
					if ((n%16) == 0) {
						if (n > 0) {
							// add the line
							strcat(s1, "\r\n");
							strcat(rxdetail_text, s1);
							strcpy(s1, "");
						}
						sprintf(t, "%04X  ", n);
						strcat(s1, t);
					}
					sprintf(t, "%02X ", (BYTE)cb.data[n]);
					strcat(s1, t);
					if (((n+1)%8) == 0 && ((n+1)%16) != 0)
						strcat(s1, "- ");
				}
				// add the last line
				strcat(s1, "\r\n");
				strcat(rxdetail_text, s1);
				SetDlgItemText(rxdhwn, 103, rxdetail_text);
				SetDlgItemInt(rxdhwn, 105, cb.buffer_length, FALSE);
			}

			break;
		default:
			strcat(s, "The action was performed successfully");
		}
		break;
	case 0x0a:
		strcat(s, "Hardware timeout error");
		break;
	case 0x10:
		strcat(s, "Change in CTS or DCD occurred");
		break;
	case 0x11:
		strcat(s, "CTS failure occurred");
		break;
	default:
		// Command not successful messages
		// If no particular return code is checked, it means that
		// there is only one left after checking above - it's not
		// very portable in the event that another return code is
		// added, but we'll need another ZTEST in that case anyway.
		switch(cb.command) {
		case BRIDGE_RX_AND_TX:
			strcat(s, "Maximum frame length exceeds 4000 bytes");
			break;
		case SET_CONFIGURATION:
			strcat(s, "Configuration data invalid");
			break;
		case RECEIVE_FRAME:
			switch(cb.return_code) {
			case 0x01:
				strcat(s, "No frames available for reception");
				break;
			case 0x03:
				strcat(s, "Receiver has not been enabled");
				break;
			}
			break;
		case SEND_FRAME:
			switch(cb.return_code) {
			case 0x01:
				strcat(s, "Transmit buffers are full");
				break;
			case 0x02:
				strcat(s, "Invalid frame length");
				break;
			case 0x03:
				strcat(s, "Communications has not been enabled");
				break;
			case 0x04:
				strcat(s, "Switched CTS/RTS not set on SET_CONFIGURATION command");
				break;
			}
			break;
		}
	}
	SAYRESPONSE(s);
}

LRESULT CALLBACK _export SelCardDlg(HWND hDlg, UINT iMessage, WPARAM wParam, LPARAM lParam)
{
	int n, m, default_card;
	LRESULT c;
	char s[5];
	static int ncard;

	switch(iMessage) {
	case WM_SYSCOMMAND:
		switch(wParam & 0xfff0) {
		case SC_CLOSE:	/* Don't allow user to close window this way */
			return(1);
		}
		break;
	case WM_INITDIALOG:
		c = ZAPIGetCards(); m = 0;
		ncard = (int)card;
		for (n=0; n < 16; n++) {
			if (c & (1L << (16+n))) {
				sprintf(s, "%2d", n);
				SendDlgItemMessage(hDlg, 101, CB_ADDSTRING, 0, (LPARAM)s);
				if (ncard == n)
					SendDlgItemMessage(hDlg, 101, CB_SETCURSEL, m, 0);
				m++;
			}
		}
		default_card = GetPrivateProfileInt(iniHardwareSection, iniCardEntry, 0, our_ini_file);
		sprintf(s, "%u", (UINT)default_card);
		SetDlgItemText(hDlg, 103, s);
		CheckDlgButton(hDlg, 104, askonstartup);
		return(1L);
	case WM_COMMAND:
		switch(GET_WM_COMMAND_ID(wParam, lParam)) {
		case 101:
			if (GET_WM_COMMAND_CMD(wParam, lParam) == CBN_SELENDOK) {
				SendDlgItemMessage(hDlg, 101, CB_GETLBTEXT, (WPARAM)SendDlgItemMessage(hDlg, 101, CB_GETCURSEL, 0, 0), (LPARAM)s);
				ncard = atoi(s);
			}
			break;
		case IDOK:
			if (IsDlgButtonChecked(hDlg, 102)) {
				sprintf(s, "%u", (UINT)ncard);
				WritePrivateProfileString(iniHardwareSection, iniCardEntry, s, our_ini_file);
			}
			if (IsDlgButtonChecked(hDlg, 104)) strcpy(s, "1");
			else strcpy(s, "0");
			if (askonstartup != (s[0]-'0'))  {
				WritePrivateProfileString(iniHardwareSection, iniAskEntry, s, our_ini_file);
				askonstartup = (s[0]-'0');
			}
			EndDialog(hDlg, ncard+1);
			break;
		case IDCANCEL:
			EndDialog(hDlg, 0);
			break;
		}
	}
	return(0);
}

int ParseTextToHex(int l)
{
	int i, j;
	WORD v;

	i=0; j=0;
	while (i < l) {
		sscanf(&txtext[i], "%2x", &v);
		cb.data[j] = (BYTE)v;
		j++;
		while (i < l && txtext[i] != ' ') i++;
		i++;
	}
	return(j);
}

/*
	Gets configuration settings as set in dialog box and updates
	data in memory
*/
void GetConfig(void)
{
	WORD i;

	cfg.line_speed = (BYTE)SendDlgItemMessage(exehwn, 800, CB_GETCURSEL, 0, 0L);
	cfg.max_frame_length = (WORD)GetDlgItemInt(exehwn, 801, NULL, FALSE);
	cfg.percent_tx_bfrs = (BYTE)GetDlgItemInt(exehwn, 802, NULL, FALSE);
	cfg.config_bits = 0;
	for (i=0; i < 8; i++)
		cfg.config_bits |= (IsDlgButtonChecked(exehwn, 810+i)) ? (0x01 << i) : 0x00;
	cfg.modem_bits = 0;
	for (i=0; i < 4; i++)
		cfg.modem_bits |= (IsDlgButtonChecked(exehwn, 820+i)) ? (0x01 << i) : 0x00;
	cfg.stat_bits = 0;
	for (i=0; i < 5; i++)
		cfg.stat_bits |= (IsDlgButtonChecked(exehwn, 830+i)) ? (0x01 << i) : 0x00;
	cfg.bit_times_open = (BYTE)GetDlgItemInt(exehwn, 803, NULL, FALSE);
	cfg.bit_times_close = (BYTE)GetDlgItemInt(exehwn, 804, NULL, FALSE);
	cfg.cts_rts_delay = (BYTE)GetDlgItemInt(exehwn, 805, NULL, FALSE);
}

/*
	Fills in dialog box with configuration settings in memory
*/
void PutConfig(void)
{
	WORD i;

	SendDlgItemMessage(exehwn, 800, CB_SETCURSEL, cfg.line_speed, 0L);
	SetDlgItemInt(exehwn, 801, cfg.max_frame_length, FALSE);
	SetDlgItemInt(exehwn, 802, cfg.percent_tx_bfrs, FALSE);
	for (i=0; i < 8; i++)
		CheckDlgButton(exehwn, 810+i, (cfg.config_bits & (0x01 << i)) ? 1 : 0);
	for (i=0; i < 4; i++)
		CheckDlgButton(exehwn, 820+i, (cfg.modem_bits & (0x01 << i)) ? 1 : 0);
	for (i=0; i < 5; i++)
		CheckDlgButton(exehwn, 830+i, (cfg.stat_bits & (0x01 << i)) ? 1 : 0);

	SetDlgItemInt(exehwn, 803, cfg.bit_times_open, FALSE);
	SetDlgItemInt(exehwn, 804, cfg.bit_times_close, FALSE);
	SetDlgItemInt(exehwn, 805, cfg.cts_rts_delay, FALSE);
}

void DoReadMany(void)
{
	// It's generally more efficient to check for data first in this
	// manner because the check is much faster than performing the
	// receive command, unless it is expected that data is present
	// because of a rapid stream of incoming frames.
	if (ZAPIqDataToRead(card)) {
		cb.command = RECEIVE_FRAME;
		cb.buffer_length = 0;
		DoCommand();
	}
}

void DoWriteString(void)
{
	cb.command = SEND_FRAME;
	cb.buffer_length = write_frame_length;
	cb.misc_tx_rx_bits = write_bits;
	memset((void *)cb.data, write_byte, write_frame_length);
	DoCommand();
	if (cb.return_code == 0) {
		if (num_write_frames == 1) {
			DoStopWrite();
		} else {
			// allow for initial setting of 0 to mean never stop
			if (num_write_frames > 0) num_write_frames--;
			if (write_inc_flag) write_byte++;
			SetDlgItemInt(exehwn, 914, num_write_frames, FALSE);
		}
	}

}

void DoStopWrite(void)
{
	writestringflag = 0;
	SetDlgItemText(exehwn, 212, szwritestring);
	EnableWindow(GetDlgItem(exehwn, 914), TRUE);
	SetDlgItemInt(exehwn, 914, initial_write_frames, FALSE);
}

LRESULT CALLBACK _export RxDetailDlg(HWND hDlg, UINT iMessage, WPARAM wParam, LPARAM lParam)
{

	switch(iMessage) {
	case WM_SYSCOMMAND:
		switch(wParam & 0xfff0) {
		case SC_CLOSE:	/* Don't allow user to close window this way */
			return(1);
		}
		break;
	case WM_INITDIALOG:
		return(1L);
	case WM_COMMAND:
		switch(GET_WM_COMMAND_ID(wParam, lParam)) {
		case 102:
			EndDialog(hDlg, 0);
			rxdhwn = NULL;
			break;
		}
	}
	return(0);
}

LRESULT CALLBACK _export StatsDlg(HWND hDlg, UINT iMessage, WPARAM wParam, LPARAM lParam)
{

	switch(iMessage) {
	case WM_SYSCOMMAND:
		switch(wParam & 0xfff0) {
		case SC_CLOSE:	/* Don't allow user to close window this way */
			return(1);
		}
		break;
	case WM_INITDIALOG:
		return(1L);
	case WM_COMMAND:
		switch(GET_WM_COMMAND_ID(wParam, lParam)) {
		case 600:
			KillTimer(statshwn, 1);
			EndDialog(hDlg, 0);
			statshwn = NULL;
			break;
		case 601:
			UpdateAllStats();
			break;
		case 500:	// flush comms errs
			cb.command = FLUSH_COMMS_ERR_STATISTICS;
			cb.buffer_length = 0;
			DoCommand();
			break;
		case 501:	// flush op stats
			cb.command = FLUSH_OPERATIONAL_STATISTICS;
			cb.buffer_length = 0;
			DoCommand();
			break;
		}
		break;

	case WM_TIMER:
		UpdateAllStats();
		break;
	}
	return(0);
}

void WriteConfig(void)
{
	OPENFILENAME ofn;
	char filebuffer[128];
	int handle;

	memset(&ofn, 0, sizeof(OPENFILENAME));
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.hwndOwner = exehwn;
	ofn.lpstrFilter = logfilter;
	ofn.nFilterIndex = 1;
	ofn.lpstrFile = filebuffer;
	ofn.nMaxFile = 128;
	ofn.lpstrTitle = "Save Configuration...";
	ofn.Flags = OFN_PATHMUSTEXIST | OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT;
	strcpy(filebuffer, "stream.cfg");

	if (GetSaveFileName(&ofn)) {
		GetConfig();
		handle = open(filebuffer, O_CREAT | O_WRONLY | O_TRUNC | O_BINARY, S_IWRITE);
		if (handle == -1) {
			MessageBox(exehwn, "Unable to open file for writing!", "File Open Error", MB_OK | MB_ICONHAND);
			return;
		}
		// start writing config to file
		write(handle, &cfg.line_speed, sizeof(cfg));
		close(handle);
	}
}

void LoadConfig(void)
{
	OPENFILENAME ofn;
	char filebuffer[128];
	int handle;

	memset(&ofn, 0, sizeof(OPENFILENAME));
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.hwndOwner = exehwn;
	ofn.lpstrFilter = logfilter;
	ofn.nFilterIndex = 1;
	ofn.lpstrFile = filebuffer;
	ofn.nMaxFile = 128;
	ofn.lpstrTitle = "Load Configuration...";
	ofn.Flags = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY;
	strcpy(filebuffer, "stream.cfg");

	if (GetOpenFileName(&ofn)) {
		GetConfig();
		handle = open(filebuffer, O_RDONLY | O_BINARY);
		if (handle == -1) {
			MessageBox(exehwn, "Unable to open file for reading!", "File Open Error", MB_OK | MB_ICONHAND);
			return;
		}
		// get config from file
		read(handle, &cfg.line_speed, sizeof(cfg));
		close(handle);
		PutConfig();
	}
}

void UpdateAllStats(void)
{
	WORD i, j;
	char s[20];
	unsigned long l;

	cb.command = READ_COMMS_ERR_STATISTICS;
	cb.buffer_length = 0;
	DoCommand();
	if (cb.return_code == 0) {
		for (i=0; i < 10; i++)
			SetDlgItemInt(statshwn, 100+i, cb.data[i], FALSE);
	}
	cb.command = READ_OPERATIONAL_STATISTICS;
	cb.buffer_length = 0;
	DoCommand();
	if (cb.return_code == 0) {
		for (j=0; j < 3; j++) {
			i = *(WORD *)(cb.data+(j<<1));
			SetDlgItemInt(statshwn, 200+j, i, FALSE);
		}
		// rest are longs
		for (j=0; j < 9; j++) {
			l = *(unsigned long *)(cb.data+6+(j<<2));
			sprintf(s, "%lu", l);
			SetDlgItemText(statshwn, 203+j, s);
		}
	}
}

